==> GeneticsBase_1.0.0.tar.gz: GeneticsBase/src/convert.cpp <==
#import "convert.h"

SEXP convert2(SEXP source, SEXP levels, SEXP byrow)
{
  int byrowflag;          // convenience alias for byrow
  int nsource, nchar, max_nchar, maxlen, clen; // iter ranges
  SEXP retval;            // return value
  int istr, ichr, j, vec; // iterators
  char *source_str, c;    // convenience tmp vars
  int val;                // "
  int nrow, ncol;         // matrix dimensions
  
  /* Check parameter types */
  if( !isString(source) || LENGTH(source)<1 )
    error("source must be a vector of type character");

  if( !isString(levels) || LENGTH(levels)<1 )
    error("levels must be a vector of type character");

  if( !isLogical(byrow) || LENGTH(byrow)!=1 )
    error("byrow must a length 1 vector be of type logical");

  /* for convenience */
  byrowflag = INTEGER(byrow)[0];

  /* Iterator ranges */
  nsource = LENGTH(source);
  clen = LENGTH(levels);
  max_nchar = 0; 

  /* Find legnth of longest element of source */
  for(vec=0; vec<nsource; vec++)
    {
      nchar = strlen( CHAR(STRING_ELT(source, 0)) );
      if(nchar > max_nchar) max_nchar = nchar;
    }
  
  if(byrowflag)
    {
      nrow = nsource;
      ncol = max_nchar;
    }
  else
    {
      nrow = max_nchar;
      ncol = nsource;
    }
  
  /* Allocate a matrix to hold the results */
  PROTECT(retval = allocMatrix(INTSXP, nrow, ncol));

  /* Iterate over source list of strings */
  for(istr=0; istr<nsource; istr++)  
    {
      R_CheckUserInterrupt(); // Check if an interrupt has occured

      /* Get the string to  work on */
      source_str = CHAR(STRING_ELT(source, istr));
      nchar = strlen(source_str);

      /* iterate over the characters */
      for(ichr=0; ichr<max_nchar; ichr++) 
        {
          val = NA_INTEGER; /* default value if no match */
          
          if(ichr < nchar)
            {
              c = source_str[ichr];
              
              /* look for a match */
              for(j=0; j<clen; j++) 
                if(c == CHAR(STRING_ELT(levels,j))[0])
                  {
                    /* found it, store and stop looking */
                    val = j+1; // R likes indexing on 1..n 
                    break;
                  }
            }
          
          if(byrowflag)
            INTEGER(retval)[istr + ichr*nrow] = val;
          else
            INTEGER(retval)[ichr + istr*nrow] = val;
        }
    }

  UNPROTECT(1);

  return retval;
  }

==> GeneticsBase_1.14.0.tar.gz: GeneticsBase/src/convert.cpp <==
#import "convert.h"

SEXP convert2(SEXP source, SEXP levels, SEXP byrow)
{
  int byrowflag;          // convenience alias for byrow
  int nsource, nchar, max_nchar, clen; // iter ranges
  SEXP retval;            // return value
  int istr, ichr, j, vec; // iterators
  //char *source_str, c;    // convenience tmp vars
  char const *source_str;    // convenience tmp vars
  char c;    // convenience tmp vars
  int val;                // "
  int nrow, ncol;         // matrix dimensions
  
  /* Check parameter types */
  if( !isString(source) || LENGTH(source)<1 )
    error("source must be a vector of type character");

  if( !isString(levels) || LENGTH(levels)<1 )
    error("levels must be a vector of type character");

  if( !isLogical(byrow) || LENGTH(byrow)!=1 )
    error("byrow must a length 1 vector be of type logical");

  /* for convenience */
  byrowflag = INTEGER(byrow)[0];

  /* Iterator ranges */
  nsource = LENGTH(source);
  clen = LENGTH(levels);
  max_nchar = 0; 

  /* Find legnth of longest element of source */
  for(vec=0; vec<nsource; vec++)
    {
      nchar = strlen( CHAR(STRING_ELT(source, 0)) );
      if(nchar > max_nchar) max_nchar = nchar;
    }
  
  if(byrowflag)
    {
      nrow = nsource;
      ncol = max_nchar;
    }
  else
    {
      nrow = max_nchar;
      ncol = nsource;
    }
  
  /* Allocate a matrix to hold the results */
  PROTECT(retval = allocMatrix(INTSXP, nrow, ncol));

  /* Iterate over source list of strings */
  for(istr=0; istr<nsource; istr++)  
    {
      R_CheckUserInterrupt(); // Check if an interrupt has occured

      /* Get the string to  work on */
      source_str = CHAR(STRING_ELT(source, istr));
      nchar = strlen(source_str);

      /* iterate over the characters */
      for(ichr=0; ichr<max_nchar; ichr++) 
        {
          val = NA_INTEGER; /* default value if no match */
          
          if(ichr < nchar)
            {
              c = source_str[ichr];
              
              /* look for a match */
              for(j=0; j<clen; j++) 
                if(c == CHAR(STRING_ELT(levels,j))[0])
                  {
                    /* found it, store and stop looking */
                    val = j+1; // R likes indexing on 1..n 
                    break;
                  }
            }
          
          if(byrowflag)
            INTEGER(retval)[istr + ichr*nrow] = val;
          else
            INTEGER(retval)[ichr + istr*nrow] = val;
        }
    }

  UNPROTECT(1);

  return retval;
  }
